package net.sf.javaprinciples.resource;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;


public class MemoryResource<T> extends AbstractResource<T>
{
    /**
     * {@inheritDoc}
     */
    @Override
    public long getResourceLength()
    {
        return content.toString().length();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public InputStream getInputStream() throws IOException
    {
        return new ByteArrayInputStream(content.toString().getBytes());
    }

    /**
     * Set the mime type.
     * @param mimeType - mime type.
     */
    public void setMimeType(String mimeType)
    {
        this.mimeType = mimeType;
    }

    // Need to implement the below in order for EasyMock-based tests to work since we don't
    // have a ready mechanism to mock the Resource being created internally to ResourceController.
    /**
     * {@inheritDoc}
     */
    @Override
    @SuppressWarnings("unchecked")
    public boolean equals(Object other)
    {
        if (!(other instanceof MemoryResource<?>))
        {
            return false;
        }
        MemoryResource<T> toCompare = (MemoryResource<T>)other;
        boolean equals = mimeType == null ? toCompare.mimeType == null : mimeType.equals(toCompare.mimeType);
        equals = equals && content == null ? toCompare.content == null : content.equals(toCompare.content);
        equals = equals && toCompare.validFrom == validFrom && toCompare.validUntil == validUntil;
        return equals;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode()
    {
        int code = 17;
        code = code * 32 + mimeType.hashCode();
        code = code * 32 + content.hashCode();
        code = code * 32 + (int)validFrom;
        code = code * 32 + (int)validUntil;
        return code;
    }

    public void setEncoding(String encoding)
    {
        this.encoding = encoding;
    }
}
