package net.sf.javaprinciples.resource;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;

import org.springframework.util.StringUtils;

public class RequestParser
{
    /**
     * Parse the parameters from the request.
     * @param request Standard J2EE request 
     * @return never null, may be empty.
     */
    public Map<String, String> parseQueryParams(HttpServletRequest request)
    {
        String queryString = request.getQueryString();
        Map<String, String> queryParameters = new HashMap<String, String>();
        if (!StringUtils.hasText(queryString))
        {
            return queryParameters;
        }
        try
        {
            String encoding = request.getCharacterEncoding();
            if (!StringUtils.hasText(encoding))
            {
                encoding = "UTF-8";
            }
            String decodedParams = URLDecoder.decode(queryString, encoding);
            String[] paramElements = decodedParams.split("&");
            for (String param : paramElements)
            {
                parseSingleParameter(queryParameters, param);
            }
        }
        catch (UnsupportedEncodingException uee)
        {
//            Loggers.validation.itil2("{2} {3}", "Unsupported URL encoding", uee.getMessage());
        }
        return queryParameters;
    }

    private void parseSingleParameter(Map<String, String> queryParameters, String param)
    {
        String[] paramPieces = param.split("=");
        queryParameters.put(paramPieces[0], paramPieces[1]);
    }

    /**
     * Parse the URI to create a list of tokens.
     * @param request Standard J2EE request 
     * @return never null but may be empty.
     */
    public List<String> parseUri(HttpServletRequest request)
    {
        List<String> pathElements = new ArrayList<String>();
        
        String resourceUri = request.getPathInfo();
        if (!StringUtils.hasText(resourceUri))
        {
            return pathElements;
        }
        StringTokenizer tokens = new StringTokenizer(resourceUri, "/");
        
        while(tokens.hasMoreTokens())
        {
            String s = tokens.nextToken();
            try
            {
                // GWT encodes URLs with UTF-8
                s = URLDecoder.decode(s, "UTF-8");
            }
            catch (UnsupportedEncodingException ignore)
            {
            }
            pathElements.add(s);
        }
        return pathElements;
    }
}
